package main

import (
	"context"
	"flag"
	"io"
	"log"
	"os"
	"net"
	"encoding/binary"
	"sync"
)

func dieif(err error) {
	if err != nil {
		log.Output(2, err.Error())
		os.Exit(1)
	}
}

func main() {
	fsrv := flag.Bool("s", false, "start lat_tcp server")
	flag.Parse()

	if *fsrv {
		serveMain()
		return
	}

	log.Fatal("lat_tcp.go: client: TODO not implemented")
}

// `lat_tcp -s`
func serveMain() {
	l, err := net.Listen("tcp", ":31234")
	dieif(err)

	// cancel = close l and indicate we are done
	ctx, ctxCancel := context.WithCancel(context.Background())
	var once sync.Once
	cancel := func() {
		once.Do(func() {
			log.Print("exit")
			ctxCancel()
			err := l.Close()
			dieif(err)
		})
	}

	for {
		conn, err := l.Accept()
		if ctx.Err() != nil {
			return
		}
		dieif(err)

		log.Printf("accepted: %v\n", conn.RemoteAddr())
		go serve(cancel, conn)
	}
}

func serve(cancel func(), conn net.Conn) {
	var msize uint32
	var b [4]byte

	_, err := io.ReadFull(conn, b[:])
	if err == io.EOF {
		cancel()
		return
	}
	dieif(err)

	msize = binary.BigEndian.Uint32(b[:])
	buf := make([]byte, msize)

	for {
		_, err = io.ReadFull(conn, buf)
		if err == io.EOF {
			break
		}
		dieif(err)
		// write always tries to write in full and returns err if less
		_, err = conn.Write(buf)
		dieif(err)
	}

	err = conn.Close()
	dieif(err)
}
